package Bank_Accounts is
    
    pragma Assertion_Policy(Pre => Check);
    pragma Assertion_Policy(Post => Check);
    
    subtype Cents_Type is Integer;

    type Account_Type is record
        Balance: Cents_Type := 0;
    end record;
    
    function Get_Balance(Account: Account_Type) return Cents_Type with
        Post => Get_Balance'Result = Account.Balance;
    -- Returns the current Balance from Account.
    
    procedure Deposit(Account: in out Account_Type; Amount: Cents_Type) with
        Pre  => (Amount > 0) and then 
                (Cents_Type'Last - Amount >= Account.Balance), 
        Post => Account.Balance = Account'Old.Balance + Amount;
    -- Deposits Amount at the given Account.
    
    procedure Withdraw(Account: in out Account_Type; Amount: Cents_Type) with 
        Pre  => (Account.Balance >= Amount) and then (Amount > 0), 
        Post => (Account.Balance = Account'Old.Balance - Amount);
    -- Withdraws Amount from the given Account.

    procedure Transfer(From: in out Account_Type; 
                       To: in out Account_Type; 
                       Amount: Cents_Type) with
        Pre  => (From.Balance >= Amount) and then
                (Amount > 0) and then
                (Cents_Type'Last - Amount >= To.Balance),
        Post => (From.Balance = From'Old.Balance - Amount) and then
                (To.Balance   = To'Old.Balance   + Amount);
    -- Transfers Amount from Account From to Account To.

    procedure Put(Account: in Account_Type);
    -- Prints the current Balance of the given Account.

end Bank_Accounts;