with AUnit.Assertions;
with Bank_Accounts;
with System.Assertions;

use AUnit.Assertions;
use Bank_Accounts;

package body Bank_Accounts_Tests is
    
    procedure Register_Tests(T: in out Test) is
        use AUnit.Test_Cases.Registration;
    begin
        Register_Routine(T, Test_Get_Balance'Access, "Test_Get_Balance");
        Register_Routine(T, Test_Deposit'Access, "Test_Deposit");
        Register_Routine(T, Test_Withdraw'Access, "Test_Withdraw");
        Register_Routine(T, Test_Transfer'Access, "Test_Transfer");
    end Register_Tests;
    
    -- ---------------------------------------------------------------

    function Name(T: Test) return Test_String is
        pragma Unreferenced(T);
    begin
        return Format("Bank_Accounts_Tests");
    end Name;
    
    -- ---------------------------------------------------------------

    procedure Test_Get_Balance(T: in out Test_Cases.Test_Case'Class) is
        pragma Unreferenced(T);
        
        Account: Account_Type;
    begin
        Assert(Get_Balance(Account) = 0, 
            "Test_Get_Balance failed.");

        Deposit(Account, 10);
        Assert(Get_Balance(Account) = 10, 
            "Test_Get_Balance deposit 10 failed.");
        
        Withdraw(Account, 5);
        Assert(Get_Balance(Account) = 5, 
            "Test_Get_Balance withdraw 5 failed.");

        begin
            Withdraw(Account, 6);
            Assert(False, "Test_Get_Balance withdraw 6 raised no exception.");
        exception 
            when System.Assertions.Assert_Failure
                => return;
            when others 
                => Assert(False, 
                    "Test_Get_Balance withdraw raised incorrect exception");
        end;
    end Test_Get_Balance;
    
    -- ---------------------------------------------------------------

    procedure Test_Deposit(T: in out Test_Cases.Test_Case'Class) is
        pragma Unreferenced(T);
        
        Sum: Cents_Type := 0;
        Num_Amounts: constant Integer := 10;
        Account: Account_Type;
    begin
        for I in 1 .. Num_Amounts loop
            Deposit(Account, I);
            Sum := Sum + I;
            Assert(Get_Balance(Account) = Sum, "Test_Deposit failed.");
        end loop;
    end Test_Deposit;
    
    -- ---------------------------------------------------------------

    procedure Test_Withdraw(T: in out Test_Cases.Test_Case'Class) is
        pragma Unreferenced(T);
        
        Sum: Cents_Type := 0;
        Num_Amounts: constant Integer := 10;
        Start_Amount: constant Cents_Type := 55;
        Account: Account_Type := Account_Type'(Balance => Start_Amount);
    begin
        for I in 1 .. Num_Amounts loop
            Assert(Get_Balance(Account) = Start_Amount - Sum, 
                "Test_Withdraw failed.");
            Withdraw(Account, I);
            Sum := Sum + I;
        end loop;

        Assert(Get_Balance(Account) = Start_Amount - Sum, 
            "Test_Withdraw failed.");
    end Test_Withdraw;
    
    -- ---------------------------------------------------------------

    procedure Test_Transfer(T: in out Test_Cases.Test_Case'Class) is
        pragma Unreferenced(T);

        Amount: constant Cents_Type := 10;
        From: Account_Type := Account_Type'(Balance => 10);
        To: Account_Type := Account_Type'(Balance => 20);
    begin
        Transfer(From, To, Amount);
        Assert(Get_Balance(From) = 10 - Amount, "Test_Transfer failed.");
        Assert(Get_Balance(To) = 20 + Amount, "Test_Transfer failed.");
    end Test_Transfer;
    
end Bank_Accounts_Tests;
