package body Graph is
    
    procedure Add_Vertex(Vertex: Vertex_Type) is 
    begin
        for I In First_Index(Vertices)..Last_Index(Vertices) loop
            if Vertex = Vertices.Element(I) then
                raise Vertex_Already_In_Graph_Exception;
            end if;
        end loop;
        
        Append(Vertices, Vertex);
    end Add_Vertex;
    
    -- ---------------------------------------------------------------

    procedure Add_Edge(From: Vertex_Type; To: Vertex_Type; Weight: Integer) is
        Edge: Edge_Type;
    begin
        for I in First_Index(Edges)..Last_Index(Edges) loop
            Edge := Edges.Element(I);
            
            if Edge.From = From and Edge.To = To then
                Edge.Weight := Weight;
                Edges.Replace_Element(I, Edge);
                return;
            end if;
        end loop;
        
        Edge := Edge_Type'(From, To, Weight);
        Append(Edges, Edge);
    end Add_Edge;
    
    -- ---------------------------------------------------------------

    procedure Clear is
    begin
        Clear(Vertices);
        Clear(Edges);
    end Clear;
    
    -- ---------------------------------------------------------------

    function Get_Edge_Weight(From: Vertex_Type; To: Vertex_Type) return Integer 
    is
        Edge: Edge_Type;
    begin
        for I in First_Index(Edges)..Last_Index(Edges) loop
            Edge := Edges.Element(I);
            
            if Edge.From = From and Edge.To = To then
                return Edge.Weight;
            end if;
        end loop;
        
        raise Edge_Not_Found_Exception;
    end Get_Edge_Weight;
    
    -- ---------------------------------------------------------------

    function Has_Edge(From: Vertex_Type; To: Vertex_Type) return Boolean is
        Edge: Edge_Type;
    begin
        for I in First_Index(Edges)..Last_Index(Edges) loop
            Edge := Edges.Element(I);
            
            if Edge.From = From and Edge.To = To then
                return True;
            end if;
        end loop;
        
        return False;
    end Has_Edge;
    
    -- ---------------------------------------------------------------

    function Remove_Edge(From: Vertex_Type; To: Vertex_Type) return Boolean is
        Edge: Edge_Type;
    begin
        for I in First_Index(Edges)..Last_Index(Edges) loop
            Edge := Edges.Element(I);
            
            if Edge.From = From and Edge.To = To then
                return True;
            end if;
        end loop;
        
        return False;
    end Remove_Edge;
    
    -- ---------------------------------------------------------------

    function To_Vertex_Array return Vertex_Array is
        Result: Vertex_Array(0..Integer(Length(Vertices)) - 1);
        J: Natural := 0;
    begin
        for I in First_Index(Vertices)..Last_Index(Vertices) loop
            Result(J) := Vertices.Element(I);
            J := J + 1;
        end loop;
        
        return Result;
    end To_Vertex_Array;

end Graph;
